@@ -91,7 +91,6 @@ class FtServer(object):
         # ref 返回的data id
 
         self.sid_to_traded = {}
-        self.sid_to_traded_money = {}
         self.oid_to_traded = {}
         self.oid_to_traded_money = {}
 
@@ -229,12 +228,9 @@ class FtServer(object):
                 time_now = time.strftime("%H:%M", time.localtime())
                 if time_now == self.endtime:
                     self.req_position()
-                    self.ft_close()
-                elif time_now == "15:10":
-                    self.req_position()
-                elif time_now == "15:00":
-                    self.req_position()                  
+                    self.ft_close()                 
                 else:
+                    self.req_position()
                     self.logger.info(f"[date_change] not_closed (now){dt}")
                 time.sleep(60)
         except Exception as e:
@@ -255,7 +251,7 @@ class FtServer(object):
     def req_position(self):
         try:
             for acc in self.accounts_run:
-                
+                target_account_name = self.target_account_names[acc]
                 account_id = self.account_id[acc]
                 product_name = self.product_names[acc]
                 broker_id = self.broker_id[account_id]
@@ -264,29 +260,33 @@ class FtServer(object):
                 data = r.json()
                 self.logger.info(f"[req_position] (url){url} (data){data}")
                 #将持仓结果计入ft_position数据库
-                if len(data['data']) > 0:
-                    ft_position_collection = self.order_info_db[acc]['ft_position']
-                    remove = ft_position_collection.delete_many({})
-                    self.logger.info(f"[req_position] delete_old_position_info (remove){remove} ")
-                    for pos in data['data']:
-                        trade_acc = pos['trade_acc']
-                        ticker = pos['stock_code']
-                        total_vol = pos['total_vol']
-                        avail_vol = pos['avail_vol']
-                        lock_vol = pos['lock_vol']
-                        exchange = decode_exchange_id(pos['exchange_id']) 
-                        position_db_msg = {
-                            'trade_acc' : trade_acc,
-                            'ticker' : ticker,
-                            'total_vol' : total_vol,
-                            'avail_vol' : avail_vol,
-                            'lock_vol' : lock_vol,
-                            'update_time' : datetime.now(timezone.utc),
-                            'exchange' : exchange
-                        }
-                        query = {'ticker' : ticker}
-                        res = ft_position_collection.replace_one(query, position_db_msg, True)
-                        self.logger.info(f"[req_position] save_position_info (res){res} (position_db_msg){position_db_msg}")
+                if data['code'] == 0:
+                    if len(data['data']) > 0:
+                        ft_position_collection = self.order_info_db[acc]['ft_position']
+                        remove = ft_position_collection.delete_many({'trade_acc': account_id})
+                        self.logger.info(f"[req_position] delete_old_position_info (remove){remove} ")
+                        for pos in data['data']:
+                            trade_acc = pos['trade_acc']
+                            ticker = pos['stock_code']
+                            total_vol = pos['total_vol']
+                            avail_vol = pos['avail_vol']
+                            lock_vol = pos['lock_vol']
+                            exchange = decode_exchange_id(pos['exchange_id']) 
+                            position_db_msg = {
+                                'trade_acc' : trade_acc,
+                                'ticker' : ticker,
+                                'total_vol' : total_vol,
+                                'avail_vol' : avail_vol,
+                                'lock_vol' : lock_vol,
+                                'update_time' : datetime.now(timezone.utc),
+                                'exchange' : exchange,
+                                'target_account_name': target_account_name
+                            }
+                            query = {'ticker' : ticker, 'trade_acc' : trade_acc, 'target_account_name' : target_account_name}
+                            res = ft_position_collection.replace_one(query, position_db_msg, True)
+                            self.logger.info(f"[req_position] save_position_info (res){res} (position_db_msg){position_db_msg}")
+                else:
+                    self.logger.error(f"[req_position] failed (error_msg){data}")
         except Exception as e:
             err = traceback.format_exc()
             self.send_to_feishu(err)
@@ -382,19 +382,9 @@ class FtServer(object):
                     self.oid_to_traded[oid] += msg['trade_vol']
                 elif target_type == 'sell':
                     self.oid_to_traded[oid] -= msg['trade_vol']
-
                 # 计算成交总价
                 amt = msg['trade_vol'] * msg['trade_price']
-                if sid not in self.sid_to_traded_money:
-                    if target_type == 'buy':
-                        self.sid_to_traded_money[sid] = amt
-                    elif target_type == 'sell':
-                        self.sid_to_traded_money[sid] = 0
-                        self.sid_to_traded_money[sid] -= amt
-                elif target_type == 'buy':
-                    self.sid_to_traded_money[sid] += amt
-                elif target_type == 'sell':
-                    self.sid_to_traded_money[sid] -= amt
+
 
                 if oid not in self.oid_to_traded_money:
                     if target_type == 'buy':
@@ -500,23 +490,19 @@ class FtServer(object):
             target_accountname = order['target_account_name']
             acc = self.target_account_names_to_acc[target_accountname]
             
-            query = {'sid': sid}
+            query = {'sid': sid, 'accountName': target_accountname}
             position_collection = self.order_info_db[acc]['EquityPosition']
             position = position_collection.find_one(query)
             if position == None and msg['trade_vol'] > 0:
                 if oid not in self.oid_to_req:
                     self.logger.warning(
-                        f"[update_position] can't_find_req (sid){sid}")
+                        f"[update_position] can't_find_req (sid){sid} (oid){oid}")
                     return
                 if oid not in self.oid_to_traded:
                     self.logger.error(f"[update_position]no trade_vol (sid){sid}")
                     return
                 td_trade_vol = self.oid_to_traded[oid]
 
-                # 计算成交总价
-                if sid not in self.sid_to_traded_money:
-                    self.logger.error(f"[update_position]no trade_amt (sid){sid}")
-                    return
                 amt = self.oid_to_traded_money[oid]
 
                 ticker = order['stock_code']
@@ -673,6 +659,7 @@ class FtServer(object):
         if type(error) == ConnectionRefusedError or type(error) == ConnectionAbortedError or type(error) == ConnectionResetError or type(error) == websocket._exceptions.WebSocketConnectionClosedException:
             self.reconnect_count += 1
             self.logger.info(f"[on_error] 正在尝试第{self.reconnect_count}次重连")
+            self.send_to_feishu(error)
             if self.reconnect_count < 100:
                 self.ws.close()
                 t = threading.Thread(target=self.run_socket)
@@ -760,7 +747,11 @@ class FtServer(object):
 
     def run(self):
         while self.is_stopped == False:
-            time.sleep(10)
+            time.sleep(3)
+            text = {"topic": "Pong", "data": int(time.time()*1000)} #毫秒时间戳
+            _text = json.dumps(text)
+            self.ws.send(_text)
+            self.logger.info(f"[on_ping] send (pong_running){_text}")
             print ("running")
     # no use
     def on_pong(self, ws, message):
@@ -931,7 +922,6 @@ class FtServer(object):
                 
                 self.sid_to_traded[sid] = 0
                 self.oid_to_traded[oid] = 0
-                self.sid_to_traded_money[sid] = 0
                 self.oid_to_traded_money[oid] = 0
                 self.oid_to_ref[oid] = ref
                 self.sids.append(sid)
@@ -966,9 +956,7 @@ class FtServer(object):
                 order_msg = dict
                 order_msg['sid'] = sid
                 order_msg['status'] = 'inserted'
-                insert_query = {'sid' : sid}
-                insert_res = ft_proposed_target_collection.replace_one(insert_query, order_msg, True)
-                self.logger.info(f"[on_req_buy_order_insert] ft_proposed_target_collection (insert_res){insert_res} (order_msg){order_msg}")
+                self.insert_ft_proposed_target(acc,sid,order_msg)
                 
                 acc = self.target_account_names_to_acc[target_account_name]
                 target_collection =  self.order_info_db[acc]['target']
@@ -991,7 +979,7 @@ class FtServer(object):
     def insert_ft_proposed_target(self, acc, sid, order_msg:dict):
         try:
             ft_proposed_target_collection = self.order_info_db[acc]['ft_proposed_target']
-            insert_query = {'sid' : sid}
+            insert_query = {'sid' : sid, 'target_account_name': order_msg['target_account_name']}
             insert_res = ft_proposed_target_collection.replace_one(insert_query, order_msg, True)
             self.logger.info(f"[on_req_buy_order_insert] ft_proposed_target_collection (insert_res){insert_res} (order_msg){order_msg}")
         except Exception as e:
@@ -1176,7 +1164,6 @@ class FtServer(object):
             oid = self.gen_order_id()
             self.sid_to_traded[sid] = 0
             self.oid_to_traded[oid] = 0
-            self.sid_to_traded_money[sid] = 0
             self.oid_to_traded_money[oid] = 0
             self.sids.append(sid)
 
