@@ -8,6 +8,7 @@ import sys
 import traceback
 import getopt
 import argparse
+from pathlib import Path
 
 from pymongo import MongoClient, ASCENDING, DESCENDING
 from utils import decode_ft_flag,  decode_exchange_id
@@ -16,13 +17,14 @@ from logger import Logger
 
 class FtTool(object): 
     
-    def __init__(self, config_filename, update_trade_date, sync_pos, sync_trade, sync_add):
+    def __init__(self, config_filename, update_trade_date, sync_pos, sync_trade, sync_add, sync_trade_in_trading):
         self.load_ft_setting(config_filename)
         self.logger = Logger.get_logger(self.logname, self.log_file_path)
         self.update_trade_date = update_trade_date
         self.sync_pos = sync_pos
         self.sync_trade = sync_trade
         self.sync_add = sync_add
+        self.sync_trade_in_trading = sync_trade_in_trading
         
         try:
             self.db_client = {}
@@ -71,6 +73,7 @@ class FtTool(object):
             path = setting['log_filepath']
             self.log_file_path = path.replace('/', '\\')
             self.ft_tradecsv_path = setting['ft_tradecsv_path'].replace('/', '\\')
+            self.ft_zidan_message_path = setting['ft_zidan_message_path'].replace('/', '\\')
             self.upload_mudan_url = setting['upload_mudan_url']
             self.feishu_url = setting.get('feishu_url')
             self.cancel_url = setting['cancel_url']
@@ -90,6 +93,7 @@ class FtTool(object):
             
             self.config = {}
             self.account_id = {}
+            self.account_id_to_acc = {}
             self.product_names = {}
             self.log_account_names = {}
             self.tgnames = {}
@@ -105,6 +109,7 @@ class FtTool(object):
                 self.config[acc] = setting['accounts'][acc]
                 config = self.config[acc]
                 self.account_id[acc] = config['account_id']
+                self.account_id_to_acc[self.account_id[acc]] = acc
                 self.product_names[acc] = config['product_name']
                 self.log_account_names[acc] = config['account_name']
                 self.tgnames[acc] = config['equity_tg_name']
@@ -156,6 +161,9 @@ class FtTool(object):
         if self.sync_trade == True:
             self.logger.info(f"in get_trade")
             self.get_trade_from_csv()
+            
+        if self.sync_trade_in_trading == True:
+            self.get_trade_from_csv_in_trading()
         
         if self.sync_pos == True:
             self.date_change()
@@ -165,6 +173,11 @@ class FtTool(object):
         
     def get_trade_from_csv(self):
         try:
+            #delete old trade msgs
+            for acc in self.accounts_run:
+                ft_sync_trade_collection = self.order_info_db[acc]['ft_sync_trade']
+                del_res = ft_sync_trade_collection.delete_many({})
+                self.logger.info(f"deleted {del_res.deleted_count}tf_sync_trade_msg")
             #trade_collection = self.test_trading_log['trade'] #测试数据库
             for acc in self.accounts_run:
                 
@@ -172,7 +185,11 @@ class FtTool(object):
                 date = dt.strftime("%Y%m%d")
                 if not self.update_trade_date == date:
                     date = self.update_trade_date
-                    filter_time_array = datetime(year=2022,month=12,day=29)
+                    int_date = int(self.update_trade_date)
+                    int_day = int(int_date%100) 
+                    int_month = int((int_date/100)%100) 
+                    int_year = int(int_date/10000)
+                    filter_time_array = datetime(year=int_year,month=int_month,day=int_day)
                 else:
                     filter_time_array = datetime.today().replace(hour=0,minute=0,second=0)
                 account_id = self.account_id[acc]
@@ -181,8 +198,8 @@ class FtTool(object):
                 trade_collection = self.tradelog_db[acc]['trade']
                 ft_order_collection = self.order_info_db[acc]['ft_order']
                 ft_sync_trade_collection = self.order_info_db[acc]['ft_sync_trade']
-                del_res = ft_sync_trade_collection.delete_many({})
-                self.logger.info(f"deleted {del_res.deleted_count}tf_sync_trade_msg")
+                #del_res = ft_sync_trade_collection.delete_many({})
+                #self.logger.info(f"deleted {del_res.deleted_count}tf_sync_trade_msg")
                 self.logger.info(f"[get_trade_from_csv] get_trade_msg_from(path){path}")
                 with open(path, encoding='gbk')as f:
                     reader = csv.reader(f)
@@ -259,7 +276,114 @@ class FtTool(object):
             self.logger.error(f'[get_trade_from_csv] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
-                    
+    
+    def get_trade_from_csv_in_trading(self):
+        try:
+            csv_path =  Path(self.ft_zidan_message_path)
+            for acc in self.accounts_run:
+                product_name = self.product_names[acc]
+                trade_acc = self.account_id[acc]
+                #for test
+                
+                """
+                if product_name == "指增1号-华鑫":
+                    product_name = "华鑫API测试"
+                elif product_name == "反脆弱1号":
+                    product_name = "盖亚青柯cats测试"
+                """
+                
+                path_head = product_name + '_' + trade_acc
+                for file in csv_path.glob("*.csv"):
+                    if file.stem.startswith(path_head):
+                        trade_collection = self.tradelog_db[acc]['trade']
+                        ft_order_collection = self.order_info_db[acc]['ft_order']
+                        ft_sync_trade_collection = self.order_info_db[acc]['ft_sync_trade']
+                        del_res = ft_sync_trade_collection.delete_many({})
+                        self.logger.info(f"[get_trade_from_csv_in_trading] deleted {del_res.deleted_count}tf_sync_trade_msg")
+                        self.logger.info(f"[get_trade_from_csv_in_trading] get_trade_msg_from(path){file.name}")
+                        with open(file, encoding='utf-8-sig')as f:
+                            reader = csv.reader(f)
+                            headers = next(reader)
+                            Row = namedtuple('Row', headers)
+                            for row in reader:
+                                row = Row(*row)
+                                if str(row.报单状态) == "全部成交" or str(row.报单状态) == "部分撤单":
+                                    traded_vol = int(row.成交数量)
+                                    traded_price = float(row.成交均价)
+                                    int_ticker = int(row.证券代码)
+                                    ticker = f'{int_ticker:06d}'
+                                    mudan_id = int(row.母单ID)
+                                    query_for_exchange = {"ticker": ticker}
+                                    exchange = 0
+                                    target = self.order_info_db[acc]['ft_position'].find_one(query_for_exchange)
+                                    #self.logger.info(f"(target){target}")
+                                    if not target == None:
+                                        exchange = target['exchange']
+                                    if traded_vol > 0:
+                                        local_id = str(row.子单编号)
+                                        origin_order_query = {"mudan_id" : mudan_id}
+                                        target = ft_order_collection.find_one(origin_order_query)
+                                        oid = 0
+                                        sid = "" # f"[PR_Ver_1_0_1_HX]{row.股票代码}T20221125"
+                                        entrust_vol = 0
+                                        side = 0
+                                        accountName = ""
+                                        order_type = 215
+                                        if not target == None:
+                                            oid = target['oid']
+                                            sid = target['sid']
+                                            entrust_vol =  int(target['order_msg']['order_vol'])
+                                            bs_flag = target['order_msg']['bs_flag']
+                                            if bs_flag == 'buy':
+                                                side = 1
+                                            elif bs_flag == 'sell':
+                                                side = 2
+                                            accountName = target['order_msg']['log_accountname']
+                                            #order_type = target['order_msg']['order_type']
+                                        _dt = datetime.now()
+                                        _date = _dt.strftime("%Y-%m-%d")
+                                        timerow = row.委托时间
+                                        _timestamp = _date + ' ' + timerow
+                                        timearray = time.strptime(_timestamp, "%Y-%m-%d %H:%M:%S")
+                                        dt = datetime.fromtimestamp(time.mktime(timearray), timezone.utc)
+                                        
+                                        trade_msg = {
+                                            "trade_ref": local_id, # broker 端的交易回报 id
+                                            "oid": oid,
+                                            "ticker": ticker,
+                                            "exchange" : exchange,
+                                            "traded_vol": traded_vol, 
+                                            "traded_price": traded_price,
+                                            "order_type": order_type, 
+                                            "side": side,  # 对应 tlclient.trader.constant 包 Side 
+                                            "entrust_vol": entrust_vol,
+                                            "entrust_price": 0,
+                                            "dbTime":datetime.now(timezone.utc),
+                                            "sid": sid, # 对应订单中的 sid
+                                            "commission": 0,
+                                            "trade_time": dt,
+                                            "accountName": accountName, # 对应 account_info.account_name
+                                            }
+                                        filter_time_array = datetime.today().replace(hour=0,minute=0,second=0)
+                                        query = {"trade_ref" : local_id , "accountName" : accountName, "trade_time": { "$gt": filter_time_array}}
+
+                                        if trade_collection.find_one(query) == None:
+                                            res = trade_collection.replace_one(query, trade_msg, True)
+                                            self.logger.info(f"[get_trade_from_csv_in_trading] (res){res} (trade_msg){trade_msg}")
+                                            if ft_sync_trade_collection.find_one(query) == None:
+                                                res = ft_sync_trade_collection.replace_one(query, trade_msg, True)
+                                                self.logger.info(f"[get_trade_from_csv_in_trading] ft_sync_trade(res){res} (trade_msg){trade_msg}")
+                                        else:
+                                            self.logger.info(f"[get_trade_from_csv_in_trading] (trade_ref){local_id}exist (trade_msg){trade_msg}")
+                        os.remove(file)
+                        self.logger.info(f"[get_trade_from_csv_in_trading] delete_csv (file_name){file.name}")   
+            
+        except Exception as e:
+            err = traceback.format_exc()
+            self.logger.error(f'[get_trade_from_csv_in_trading] (exception){err}')
+            exc_type, exc_value, exc_traceback = sys.exc_info()
+            traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
+                  
     def date_change(self):
         try:
                 dt = datetime.now()
@@ -268,9 +392,9 @@ class FtTool(object):
                     self.logger.info("[date_change] date_change_open")
                     self.sync_position()
                     self.update_position_date_open()
-                elif dt.hour < self.sync_position_close:
+                elif dt.hour < self.sync_position_close and dt.hour > self.sync_position_open:
                     self.logger.info("[date_change] date_not_change")
-                    time.sleep(600)
+                    self.sync_position()
                 elif dt.hour >= self.sync_position_close and self.sync_pos == True:
                     self.logger.info("[date_change] date_change_close")
                     self.sync_position()
@@ -312,29 +436,75 @@ class FtTool(object):
                                         vol += sync_trade['traded_vol']
                                     elif sync_trade['side'] == 2:
                                         vol -= sync_trade['traded_vol']
-                            else:
+                                for key,value in sid_to_traded_vol.items():
+                                    position_sid = key
+                                    position_yd_vol = value
+                                    query_by_sid = {'sid': position_sid}
+                                    one_trade = trade_today_collection.find_one(query_by_sid)
+                                    exchange = one_trade['exchange']
+                                    position_for_update = equity_position_collection.find_one(query_by_sid)
+                                    if position_for_update is not None:
+                                        yd_pos_long = position_for_update['yd_pos_long'] + position_yd_vol
+                                        actual_yd_pos_long = position_for_update['actual_yd_pos_long'] + position_yd_vol
+                                        data_to_update ={
+                                            'yd_pos_long' : yd_pos_long,
+                                            'actual_yd_pos_long' : actual_yd_pos_long
+                                        }
+                                        new_data = {"$set": data_to_update}
+                                        res = equity_position_collection.update_one(query_by_sid, new_data)
+                                        self.logger.info(f"[sync_position_add] update_success (res){res}")
+                                    else:
+                                        self.add_position_for_add(acc, exchange = exchange, ticker = ticker, td_vol = 0, yd_vol = position_yd_vol, accountName = account_name, sid = position_sid)
+                                query_for_delete_add = {'sid' : sid , 'accountName' : account_name}
+                                equity_position_collection.delete_one(query_for_delete_add)
+                                self.logger.info(f"delete_position (sid){sid}")
+                            else: #找不到sync_trade 直接将add持仓分到所有持仓上
                                 self.logger.info("[sync_position_add] nothing")
-                            for key,value in sid_to_traded_vol.items():
-                                position_sid = key
-                                position_yd_vol = value
-                                query_by_sid = {'sid': position_sid}
-                                one_trade = trade_today_collection.find_one(query_by_sid)
-                                exchange = one_trade['exchange']
-                                position_for_update = equity_position_collection.find_one(query_by_sid)
-                                if position_for_update is not None:
-                                    yd_pos_long = position_for_update['yd_pos_long'] + position_yd_vol
-                                    actual_yd_pos_long = position_for_update['actual_yd_pos_long'] + position_yd_vol
+                                yd_pos_long = position['yd_pos_long']
+                                actual_yd_pos_long = position['actual_yd_pos_long']
+                                if actual_yd_pos_long > 0:
+                                    query_positions = {'accountName': account_name, 'ticker': ticker, 'sid' :  {'$ne' : sid}}
+                                    position_for_update = equity_position_collection.find_one(query_positions)
+                                    yd_pos_long = position_for_update['yd_pos_long'] + actual_yd_pos_long
+                                    actual_yd_pos_long = position_for_update['actual_yd_pos_long'] + actual_yd_pos_long
+                                    _id = position_for_update['_id']
+                                    _query = {'_id': _id}
                                     data_to_update ={
                                         'yd_pos_long' : yd_pos_long,
                                         'actual_yd_pos_long' : actual_yd_pos_long
                                     }
                                     new_data = {"$set": data_to_update}
-                                    res = equity_position_collection.update_one(query_by_sid, new_data)
-                                else:
-                                    self.add_position_for_add(acc, exchange = exchange, ticker = ticker, td_vol = 0, yd_vol = position_yd_vol, accountName = account_name, sid = position_sid)
-                            query_for_delete_add = {'sid' : sid}
-                            equity_position_collection.delete_one(query_for_delete_add)
-                            self.logger.info(f"delete_position (sid){sid}")
+                                    res = equity_position_collection.update_one(_query, new_data)
+                                elif actual_yd_pos_long < 0:
+                                    query_positions = {'accountName': account_name, 'ticker': ticker, 'sid' : {'$ne' : sid}}
+                                    ticker_positions = equity_position_collection.find(query_positions)
+                                    if ticker_positions.count() > 0:
+                                        for ticker_position in ticker_positions:
+                                            pos = ticker_position['actual_yd_pos_long'] + actual_yd_pos_long
+                                            _id = ticker_position['_id']
+                                            if pos > 0 :
+                                                data_to_update ={
+                                                                    'yd_pos_long' : pos,
+                                                                    'actual_yd_pos_long' : pos
+                                                                }
+                                                new_data = {"$set": data_to_update}
+                                                query_to_update = {'_id': _id}
+                                                res = equity_position_collection.update_one(query_to_update, new_data)
+                                                self.logger.info(f"[sync_position_add] update_success (res){res}")
+                                                
+                                                query_for_delete_add = {'sid' : sid , 'accountName' : account_name}
+                                                equity_position_collection.delete_one(query_for_delete_add)
+                                                self.logger.info(f"delete_position (sid){sid}")
+                                                break
+                                            elif pos <= 0:
+                                                query_for_delete = {'_id': _id}
+                                                res = equity_position_collection.delete_one(query_for_delete)
+                                                self.logger.info(f'[sync_position_add] delete (res){res} ')
+                                                yd_pos_long = pos
+                                                
+                                query_for_delete_add = {'sid' : sid , 'accountName' : account_name}
+                                equity_position_collection.delete_one(query_for_delete_add)
+                                self.logger.info(f"delete_position (sid){sid}")
                             
         except Exception as e:
             err = traceback.format_exc()
@@ -369,7 +539,7 @@ class FtTool(object):
             record['holding_days'] = 0 
             record['sid'] = sid
             record['exchange'] = exchange
-            filter = {'sid' : record['sid']}
+            filter = {'sid' : record['sid'], 'accountName' : accountName}
             res = collection.replace_one(filter, record, True) 
             self.logger.info(f"[add_position_for_debug] (res){res} (msg){record}")
         except Exception as e:
@@ -390,7 +560,7 @@ class FtTool(object):
             for position in targets:
                         self.logger.info(f"goes in (position){position}")
                         sid = position['sid']
-                        query = {'sid' : sid}
+                        query = {'sid' : sid, 'accountName' : target_accountname}
                         holdingdays = position['holding_days']
                         yd_pos = position['actual_td_pos_long'] + position['actual_yd_pos_long']
                         td_pos = 0
@@ -416,7 +586,7 @@ class FtTool(object):
                 continue
             for position in targets:
                     sid = position['sid']
-                    query = {'sid' : sid}
+                    query = {'sid' : sid, 'accountName': target_accountname}
                     holdingdays = position['holding_days'] + 1
                     yd_pos = position['actual_td_pos_long'] + position['actual_yd_pos_long']
                     td_pos = 0
@@ -475,9 +645,9 @@ class FtTool(object):
                                     sync_yd_vol = yd_vol - yd_vol_indb 
                                     self.add_position_for_debug(acc, pos, ticker, sync_td_vol, sync_yd_vol ,accountName)
                 else:
-                    self.logger.warning("[sync_position]can't_find_positions")
+                    self.logger.warning(f"[sync_position]can't_find_positions (trade_acc){account_id}")
                                 
-                self.logger.info("[sync_position] sync_position_finished")
+                self.logger.info(f"[sync_position] sync_position_finished (trade_acc){account_id}")
             
         except Exception as e:
             err = traceback.format_exc()
@@ -489,7 +659,7 @@ class FtTool(object):
         try:
             collection = self.order_info_db[acc]['EquityPosition']
             sid = target['sid']
-            update_filter = {'sid': sid}
+            update_filter = {'sid': sid, 'accountName' : accountName}
             update_record = {
                 'td_pos_long': td_vol,
                 "yd_pos_long": yd_vol,
@@ -532,7 +702,7 @@ class FtTool(object):
             record['holding_days'] = 0 
             record['sid'] = '[ADD]' + self.generate_sid(ticker)
             record['exchange'] = exchange
-            filter = {'sid' : record['sid']}
+            filter = {'sid' : record['sid'], 'accountName': accountName}
             res = collection.replace_one(filter, record, True) 
             self.logger.info(f"[add_position_for_debug] (res){res} (msg){record}")
         except Exception as e:
@@ -570,10 +740,11 @@ if __name__ == "__main__":
     parser.add_argument('-t', '--sync_trade', dest='sync_trade', type=str2bool, default='T')
     parser.add_argument('-p', '--sync_pos', dest='sync_pos', type=str2bool, default='T')
     parser.add_argument('-a', '--sync_add', dest='sync_add', type=str2bool, default='F')
+    parser.add_argument('-i', '--sync_trade_in_trading', dest='sync_trade_in_trading',type=str2bool, default='F')
     
     args = parser.parse_args()
     print (f"(args){args}")
 
-    td = FtTool(args.config_filename, args.update_trade_date, args.sync_pos, args.sync_trade, args.sync_add)
+    td = FtTool(args.config_filename, args.update_trade_date, args.sync_pos, args.sync_trade, args.sync_add, args.sync_trade_in_trading)
     
     td.start()  
